#include<iostream>
#include<fstream>
#include<iomanip>
#include<string>
#include<cstring>
#include<cmath>
#include"globvar.h" // some variables and function declarations
#include"rng.h"
#include<ctime>

using namespace std;

cRNG seed(7654321);

double r(void)
{
    return seed.Randf();
}

string mkDNA(int length)
{
    double temp;
    string str = "A"; // first base doesn't matter, by symmetry...
    for(int i = 0; i<length-1; i++)
    {
	temp = r();
	if(temp < 0.25) str.append("A");
	else if(temp < 0.5) str.append("C");
	else if(temp < 0.75) str.append("G");
	else str.append("T");
    }

    // Use the following code to create self-symmetric strands (i.e., strands that are complementary to itself as a whole)
   /* 
    str.append("X");
    for(int i = length-1; i>=0; i--)
    {
	if(str.at(i) == 'A') str.append("T");
	else if(str.at(i) == 'C') str.append("G");
	else if(str.at(i) == 'G') str.append("C");
	else str.append("A");
    }*/
    return str;
}


#define die(s) {printf(s); exit(1);} // Abort program, displaying the message (s)... (Use for fatal errors)
#define COUNTNUM 9 // The number of counters in the runTest function.


// =====================================================
//   TRANSLATE FUNCTION (from Partsum_NN.cpp)

/*	The translate(DNAstrand) function will change
	the chars in the strand to the values 0,1,2,3
	meaning A,C,G,T for DNA, and 4,5,6,7 meaning
	A,C,G,U for RNA.
	This will allow for a quicker calculation of the
	partition sum. These are dynamically allocated,
	to allow any strand length. Possibly, the memory
	cost can be cut down further by replacing short
	ints by something smaller, as I only need the
	integers 0,1,2,3,4,5,6 and 7. However, it should
	allow enumeration, to take some element from the
	binding energy matrix. Perhaps the "enum"
	statement solves this, but at the time of
	writing, this improvement seems hardly relevant.
	Moreover, the memory cost is only of order $N$,
	so these considerations do not seem very
	relevant.
*/
// =====================================================

void translate(string strand, short int * numstr)
{
    int i; // counter
    int strlength = strand.length();

    for(i=0;i<strlength;i++)
    {
	numstr[i] = 8;

	if(strand.at(i) == 'A') numstr[i] = 0;
	if(strand.at(i) == 'C') numstr[i] = 1;
	if(strand.at(i) == 'G') numstr[i] = 2;
	if(strand.at(i) == 'T') numstr[i] = 3;
	if(strand.at(i) == 'B') numstr[i] = 4;
	if(strand.at(i) == 'D') numstr[i] = 5;
	if(strand.at(i) == 'H') numstr[i] = 6;
	if(strand.at(i) == 'U') numstr[i] = 7;
	if(strand.at(i) == 'X') numstr[i] = 15;

	if(numstr[i] == 8)
	{
	    cout << "FATAL ERROR!\n Invalid character '"<< strand.at(i) << "' encountered!\n\n";
	    exit(1);
	}
    }
}
// =====================================================
//   end TRANSLATE FUNCTION
// =====================================================

// =====================================================
//  REDUCTION MATRIX FUNCTION (redmat)
// (child function of Recursive Partsum function)

/* This function uses a matrix to help reduce reducible
   partial partition sums, into irreducible once. In the
   latter, a partial partition sum is said to be
   IRREDUCIBLE if and only if it evaluates to a greater
   value than any of its contained partial partition
   sums. */

// Function declaration for use in functions to come
double ppsum(short int *, int, int, int, double **, short int **,
	int **, int **, double **, double **);

double ppsumarc(short int *, int, int, int, double **, short int **,
	int **, int **, double **, double **);

// Partial partition sum using reduction matrix (do it inline!)
inline double ppsumred(short int * strand, int strlength, int i /*start*/,
	int j /*length*/, double ** expbdg, short int ** redmat,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    if(matr[i][j] > 0) return matr[i][j]; // For redmat[i][2*j] == -1, matr[i][j] = 1 has already been done
    else if(redmat[i][2*j] == -1) {matr[i][j] = 1; return 1;} // Trivial partition sum, return 1
    else if(redmat[i][2*j] < -1) // Irreducible?
    { // Then we'll just have to do the calculation
	matr[i][j] = ppsum(strand, strlength, i, j, expbdg,
		redmat, offset, kliststart, matr, matrbow);
	return matr[i][j];
    }
    
    else if(matr[redmat[i][2*j]][redmat[i][2*j+1]] > 0) // Already done the reduced one? 
    {
	if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] == 0) return matr[redmat[i][2*j]][redmat[i][2*j+1]];
	else if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] > 1) return matrbow[redmat[i][2*j]][redmat[i][2*j+1]] + matr[redmat[i][2*j]][redmat[i][2*j+1]];
	else
	{
	    matrbow[redmat[i][2*j]][redmat[i][2*j+1]] = ppsumarc(strand, strlength, redmat[i][2*j], redmat[i][2*j+1], expbdg,
	       	redmat, offset, kliststart, matr, matrbow);
	    return matrbow[redmat[i][2*j]][redmat[i][2*j+1]] + matr[redmat[i][2*j]][redmat[i][2*j+1]];
	}
	
	matr[i][j] = matr[redmat[i][2*j]][redmat[i][2*j+1]]; // Would it be good to also write all intermediate reduction ppsums? Or is this only extra work?
	if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] > 1) matr[i][j] += matrbow[redmat[i][2*j]][redmat[i][2*j+1]];
	if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] == 1)
	{
	    matrbow[redmat[i][2*j]][redmat[i][2*j+1]] = ppsumarc(strand, strlength, redmat[i][2*j], redmat[i][2*j+1], expbdg,
	       	redmat, offset, kliststart, matr, matrbow);
	    matr[i][j] += matrbow[redmat[i][2*j]][redmat[i][2*j+1]];
	}
	return matr[i][j];
	
    }
    else // Haven't done this irreducible -but necessary- ppsum yet, so do it now
    {
	matr[redmat[i][2*j]][redmat[i][2*j+1]] = ppsum(strand, strlength, redmat[i][2*j], redmat[i][2*j+1], expbdg,
		redmat, offset, kliststart, matr, matrbow);
	matr[i][j] = matr[redmat[i][2*j]][redmat[i][2*j+1]];
	if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] > 1) matr[i][j] += matrbow[redmat[i][2*j]][redmat[i][2*j+1]];
	if(matrbow[redmat[i][2*j]][redmat[i][2*j+1]] == 1)
	{
	    matrbow[redmat[i][2*j]][redmat[i][2*j+1]] = ppsumarc(strand, strlength, redmat[i][2*j], redmat[i][2*j+1], expbdg,
	       	redmat, offset, kliststart, matr, matrbow);
	    matr[i][j] += matrbow[redmat[i][2*j]][redmat[i][2*j+1]];
	}
	return matr[i][j];
    }
    
}

// =====================================================
//   PARTIAL PARTITION SUM WITH START-END ARCS (ppsumarc)
// (child function of Recursive Partsum function)

/*  This function calculates a special case of partial
    parition sums, namely, the part of the partial
    partition sum which consists of all configurations
    that have base i (the first) and base i+j-1 (the
    last) connected.

    Call this function ONLY when matrbow[i][j] != 0.
    
    Input parameters:
    strand - The strand of DNA and/or RNA as numbers 0-7
    strlength - length of the complete strand
    start - where the partial sum starts
    length - length of the partial sum (# of nucleotides)
    expbdg - energy contrib. for a bow, depends on config.
    redmat - the Reduction Matrix, reduces workload
    offset - Startpoints of lists of pairs. Required to
    	     know where to end the list of k's.
    kliststart - for every i, contains pointer to first
    		 relevant k in list of k's.
    matr - matrix containing calculated partition sums.
    matrbow - same for partsums with start-end arcing.

    Output parameters:
    result - Return the value obtained for this partition
    	     sum.
    */
// =====================================================

double ppsumarc(short int * strand, int strlength, int start,
	int length, double ** expbdg, short int ** redmat,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    if(length<4+MINBOW) return 0; // No bow, so NO contribution!
    double innerbow;
    if(length > MINBOW + 8) // Nontrivial innerbow possible
	innerbow = ppsumred(strand, strlength, start+2, length-4, expbdg,
	    redmat, offset, kliststart, matr, matrbow);
    else innerbow = 1;
    if(matrbow[start+1][length-2] == 1)
	matrbow[start+1][length-2] = ppsumarc(strand, strlength, start+1,
		length-2, expbdg, redmat, offset, kliststart, matr, matrbow);
    innerbow += matrbow[start+1][length-2];
    
    // Return the outcome.
    return innerbow*expbdg[strand[start]][strand[start+length-2]];
}


// =====================================================
//   PARTIAL PARTITION SUM FUNCTION (ppsum)
// (child function of Recursive Partsum function)

/*  This function calculates a partial partition sum.
    It calls itself, and the partial partition sum with
    a start-end arc named ppsumarc.

    Note to self: In case that turns out to decrease
    computation time, a lot of if(matr[i][j] == 1) {...}
    statements may be inserted.
 
    N.B. This result does NOT include start-end arcing.
    For this, use the ppsumarc function.

    Input parameters:
    strand - The strand of DNA and/or RNA as numbers 0-7
    strlength - length of the complete strand
    start - where the partial sum starts
    length - length of the partial sum (# of nucleotides)
    expbdg - energy contrib. for a bow, depends on config.
    redmat - the Reduction Matrix, reduces workload
    offset - Startpoints of lists of pairs. Required to
    	     know where to end the list of k's.
    kliststart - for every i, contains pointer to first
    		 relevant k in list of k's.
    matr - matrix containing calculated partition sums.
    matrbow - same for partsums with start-end arcing.

    Output parameters:
    result - Return the value obtained for this partition
    	     sum.
    */

// =====================================================

double ppsum(short int * strand, int strlength, int start,
	int length, double ** expbdg, short int ** redmat,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    int k;
    int * ptr;
    int i = start; // Start position of partial partition sum
    int j = length; // Length of partial partition sum (# of nucleotides)
    double result; // Result to be returned
    double extarc; // Replaces 'prebow'
    double intarc; // Replaces 'innerbow'
    result = ppsumred(strand, strlength, i+1, j-1, expbdg,
		redmat, offset, kliststart, matr, matrbow);
    // Find out whether bow has been calc'ed before:
    if(matrbow[i+1][j-1] == 1)
    {
	matrbow[i+1][j-1] = ppsumarc(strand, strlength, i+1, j-1,
		expbdg, redmat, offset, kliststart, matr, matrbow);
    }
    result += matrbow[i+1][j-1];
    // Initialization done, now do the actual work
  for(ptr=kliststart[i]; ptr<offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
    {
	k = ptr[0]+2-i; // This k is actually the length of the bow i,i+1 -> i+k-2,i+k-1.
			// +1 because the k is the first of the pair (in the list of k's).
			// We want an i -> k and i+1 -> k-1 arc.
			// But also +1 because k is the number
			// of positions, k=i_end-i_start+1.
	if(k > j-1) break; // Too far, bow exceeds partsum. Recall that partsum ends at position j-1.
	if(k < MINBOW+4) continue; // Too small bow, perhaps next bow is better
	if(matrbow[i][k] == 1)
	{
	    matrbow[i][k] = ppsumarc(strand, strlength, i, k, expbdg,
		redmat, offset, kliststart, matr, matrbow);
	}
	intarc = matrbow[i][k];
	if(j - k > 4)//3+MINBOW) // Note that the arc ends at kth pos. s.t.
	    	      // we have positions k+1, ..., j-1 left.
	{
	    extarc = ppsumred(strand, strlength, i+k, j-k, expbdg,
		redmat, offset, kliststart, matr, matrbow);
	    if(matrbow[i+k][j-k] == 1)
		matrbow[i+k][j-k] = ppsumarc(strand, strlength, i+k, j-k, expbdg,
		    redmat, offset, kliststart, matr, matrbow);
	    extarc += matrbow[i+k][j-k]; // Possibly (often) zero
	}
	else extarc = 1;
	result += intarc*extarc; // Contribution to ppsum
    }

    matr[i][j] = result;
    return result;
	    
} // End ppsum function



// =====================================================
//  Recursive PARTITION FUNCTION, main function

/*  This version of the Partition Function is an
    improved version of the previous one. Recursive
    techniques are used for a more efficient calculation.

    Input parameters:
    * strand: A strand of numbers 0-7 to represent RNA
    * strlength: The length of the Dyn. All. strand
    * expbdg: binding energy values
    * Partial_PS: array for storage of results
    * Partial_PSbow: array for storage of results with start-end arc
    * offset: table of k's sorted by nearest-neighbor pair type
    * kliststart: table of pointers into `offset' table for efficient access
    * redmat: reduction matrix
    * cleanmatrcount: Auxiliary variable, helps timing the cost of initializing the matrices

    Characteristics:
    The recursive Partsum function uses recursive
    techniques to calculate the large partition sum,
    working top-down rather than bottom-up.
    
    Method summary:
    The complete partition sum is calculated by running
    through a tree of all required partition sums. The
    two functions ppsum and ppsumarc, which calculate
    p(i,j) and pbow(i,j), respectively, call itself and
    each other repeatedly.

    Returned value:
    As is the case for all algorithms, p(0,N) and
    pbow(0,N) are computed separately, thus the return
    value is the sum of these values.
*/
// =====================================================

double recpartsum(short int * strand, int strlength, double ** expbdg, double** matr,
	double** matrbow, int** offset, int** kliststart, short int ** redmat, int cleanmatrcount)
{
    int i,j;
    double result = 0; // Store the result for the partition sum here.
    int * ptr; // Pointer as loop variable (Loop through array)
for(int k = 0; k<cleanmatrcount; k++)
{
    // INITIALIZE PARTIAL RESULT ARRAYS
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	{
	    matr[i][j] = matrbow[i][j] = 0;
	}
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    matrbow[i][ptr[0]-i+2] = 1;
	}
    }
}
    result = ppsumred(strand, strlength, 0, strlength, expbdg,
	redmat, offset, kliststart, matr, matrbow); // No-bow partsum
    if(matrbow[0][strlength] == 1)
	matrbow[0][strlength] = ppsumarc(strand, strlength, 0, strlength,
		expbdg, redmat, offset, kliststart, matr, matrbow);
    result += matrbow[0][strlength]; // start-end arc partsum

    return result;
} // end recursive partition sum

// ==============================================================
// ============== Recursive algo without Redmat =================
// ==============================================================

double ppsum(short int *, int, int, int, double**, int**, int**, double**, double**);
double ppsumarc(short int *, int, int, int, double**, int**, int**, double**, double**);

// Partial partition sum recursive auxiliary function: check whether this thing has already been computed before
inline double ppsumred(short int * strand, int strlength, int i /*start*/,
	int j /*length*/, double ** expbdg,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    if(matr[i][j] > 0) return matr[i][j];
    else
    {
	matr[i][j] = ppsum(strand, strlength, i, j, expbdg,
		offset, kliststart, matr, matrbow);
	return matr[i][j];
    }
}

// ============= Start-End Arc function without Reduction Matrix ==========

//  For a description of input parameters and method
//  summary, see the Reduction Matrix version above

double ppsumarc(short int * strand, int strlength, int start,
	int length, double ** expbdg,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    if(length<4+MINBOW) return 0; // No bow, so NO contribution!
    double innerbow;
    if(length > MINBOW + 8) // Nontrivial innerbow possible
	innerbow = ppsumred(strand, strlength, start+2, length-4, expbdg,
	    offset, kliststart, matr, matrbow);
    else innerbow = 1;
    if(matrbow[start+1][length-2] == 1)
	matrbow[start+1][length-2] = ppsumarc(strand, strlength, start+1,
		length-2, expbdg, offset, kliststart, matr, matrbow);
    innerbow += matrbow[start+1][length-2];
    
    // Return the outcome.
    return innerbow*expbdg[strand[start]][strand[start+length-2]];
}


// =====================================================
//   PARTIAL PARTITION SUM FUNCTION (ppsum)
// (child function of Recursive Partsum function)

/*  This function calculates a partial partition sum.
    It calls itself, and the partial partition sum with
    a start-end arc named ppsumarc.

    Note to self: In case that turns out to decrease
    computation time, a lot of if(matr[i][j] == 1) {...}
    statements may be inserted.
 
    N.B. This result does NOT include start-end arcing.
    For this, use the ppsumarc function.

    Input parameters:
    strand - The strand of DNA and/or RNA as numbers 0-7
    strlength - length of the complete strand
    start - where the partial sum starts
    length - length of the partial sum (# of nucleotides)
    expbdg - energy contrib. for a bow, depends on config.
    offset - Startpoints of lists of pairs. Required to
    	     know where to end the list of k's.
    kliststart - for every i, contains pointer to first
    		 relevant k in list of k's.
    matr - matrix containing calculated partition sums.
    matrbow - same for partsums with start-end arcing.

    Output parameters:
    result - Return the value obtained for this partition
    	     sum.
    */

// =====================================================

double ppsum(short int * strand, int strlength, int start,
	int length, double ** expbdg,
	int ** offset, int ** kliststart, double ** matr, double ** matrbow)
{
    int k;
    int * ptr;
    int i = start; // Start position of partial partition sum
    int j = length; // Length of partial partition sum (# of nucleotides)
    if(j<5+MINBOW) {matr[i][j] = 1; return 1;} // Too short, no arcing possible, or only a start-end arc
    double result; // Result to be returned
    double extarc; // Replaces 'prebow'
    double intarc; // Replaces 'innerbow'
    result = ppsumred(strand, strlength, i+1, j-1, expbdg,
		offset, kliststart, matr, matrbow);
    if(strand[i+1] > 7) return result; // If part of strand starts with an X, this is all there is.
    // Find out whether bow has been calc'ed before:
    if(matrbow[i+1][j-1] == 1)
    {
	matrbow[i+1][j-1] = ppsumarc(strand, strlength, i+1, j-1,
		expbdg, offset, kliststart, matr, matrbow);
    }
    result += matrbow[i+1][j-1];
    if(strand[i] > 7) return result; // There's nothing more because we have one X
    // Initialization done, now do the actual work
  for(ptr=kliststart[i]; ptr<offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
    {
	k = ptr[0]+2-i; // This k is actually the length of the bow i,i+1 -> i+k-2,i+k-1.
			// +1 because the k is the first of the pair (in the list of k's).
			// We want an i -> k and i+1 -> k-1 arc.
			// But also +1 because k is the number
			// of positions, k=i_end-i_start+1.
	if(k > j-1) break; // Too far, bow exceeds partsum. Recall that partsum ends at position j-1.
	if(k < MINBOW+4) continue; // Too small bow, perhaps next bow is better
	if(matrbow[i][k] == 1)
	{
	    matrbow[i][k] = ppsumarc(strand, strlength, i, k, expbdg,
		offset, kliststart, matr, matrbow);
	}
	intarc = matrbow[i][k];
	if(j - k > 4)//3+MINBOW) // Note that the arc ends at kth pos. s.t.
	    	      // we have positions k+1, ..., j-1 left.
	{
	    extarc = ppsumred(strand, strlength, i+k, j-k, expbdg,
		offset, kliststart, matr, matrbow);
	    if(matrbow[i+k][j-k] == 1)
		matrbow[i+k][j-k] = ppsumarc(strand, strlength, i+k, j-k, expbdg,
		    offset, kliststart, matr, matrbow);
	    extarc += matrbow[i+k][j-k]; // Possibly (often) zero
	}
	else extarc = 1;
	result += intarc*extarc; // Contribution to ppsum
    }

    matr[i][j] = result;
    return result;
	    
} // End ppsum function

// Recursive partition sum without reduction matrix
double recpartsum(short int * strand, int strlength, double ** expbdg, double** matr,
	double** matrbow, int** offset, int** kliststart, int cleanmatrcount)
{
    int i,j;
    double result = 0; // Store the result for the partition sum here.
    int * ptr; // Pointer as loop variable (Loop through array)
for(int k = 0; k<cleanmatrcount; k++)
{
    // INITIALIZE PARTIAL RESULT ARRAYS
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	{
	    matr[i][j] = matrbow[i][j] = 0;
	}
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    matrbow[i][ptr[0]-i+2] = 1;
	}
    }
}
    result = ppsum(strand, strlength, 0, strlength, expbdg,
	offset, kliststart, matr, matrbow); // No-bow partsum
    if(matrbow[0][strlength] == 1)
	matrbow[0][strlength] = ppsumarc(strand, strlength, 0, strlength,
		expbdg, offset, kliststart, matr, matrbow);
    result += matrbow[0][strlength]; // start-end arc partsum

    return result;
    
} // end recursive partition sum

// =====================================================
//   PARTITION FUNCTION (using k-list only)

/*  This function uses only the k-list. Its only
 *  difference compared to the function hereunder is the
 *  use of the Reduction Matrix, which was only used in
 *  two subsequent lines, to prevent running any
 *  unnecessary loops. It is expected to run slower than
 *  the version which also incorporates the Reduction
 *  Matrix. For an overview of Input parameters and
 *  Method summary, see the function description for the
 *  other version.
 */
// =====================================================

double partsum(short int * strand, int strlength, double ** expbdg, double** Partial_PS,
	double** Partial_PSbow, int** offset, int** kliststart)
{
    if(MINBOW >= strlength-2) return 1;

    int i,j,k;
    k = 0; // Initialize
    int * ptr;
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    if(ptr[0]-i+2 > 3+MINBOW) Partial_PSbow[i][ptr[0]-i+2] = 1; // For future reference, a bow should come here
	}
    }

    for (j=MINBOW+3; j<strlength+1; j++) // j is the bowlength
    {
	for (i=0; i < (strlength+1 - j); i++) // i is the starting position of the i-j partition sum under consideration
	{
	    Partial_PS[i][j] = Partial_PS[i+1][j-1];	// Start with the No-Bow case, there is NO i->j bow.
	    Partial_PS[i][j] += Partial_PSbow[i+1][j-1]; // Now, we DO want to allow this bow in the partition sum
	    for(ptr=kliststart[i]; ptr<offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	    {
		k = ptr[0]+2-i; // +1 because the k is the first of the pair (in the list of k's).
				// We want an i -> k and i+1 -> k-1 arc.
				// But also +1 because k is the number
				// of positions, k=i_end-i_start+1.
		if(k > j-1) break; // Too far, bow exceeds partsum. Recall that partsum ends at position j-1.
		if(k < MINBOW+4) continue; // Too small bow, perhaps next bow is better
		Partial_PS[i][j] += Partial_PSbow[i][k]*(Partial_PS[i+k][j-k] + Partial_PSbow[i+k][j-k]);
	    }
	    if(Partial_PSbow[i][j] == 1)
	    {
		Partial_PSbow[i][j] = expbdg[strand[i]][strand[i+j-2]]*
		    (Partial_PSbow[i+1][j-2] + Partial_PS[i+2][j-4]);
	    }
	}
    }
    double partsum = Partial_PS[0][strlength] + Partial_PSbow[0][strlength];

  return partsum;
    
} // End of iterative algo with klist only 

// =====================================================
//   PARTITION FUNCTION (using k-list and reduction matrix)
/*
    Function variables:
    * strand: A strand of numbers 0-7 to represent RNA
    * strlength: The length of the Dyn. All. strand
    * expbdg: binding energy values
    * Partial_PS: array for storage of results
    * Partial_PSbow: array for storage of results with start-end arc
    * offset: table of k's sorted by nearest-neighbor pair type
    * kliststart: table of pointers into `offset' table for efficient access
    * redmat: reduction matrix

    Characteristics:
    This version of the Partsum function calculates a
    partition sum without doing any unnecessary work.
    This is achieved by using a list of pairs (to
    quickly determine what arcs are possible within a
    partition sum) and a reduction matrix (to prevent
    carrying out other unnecessary computations).

    Method summary:
    The partial result lists Partial_PS and
    Partial_PSbow are filled bottom-up, that is, the
    smallest are filled first. This means the outermost
    loop is a loop over the length, j, then we loop over
    all possible positions i.
    First, the Reduction Matrix is checked to see if the
    current Partial_PS will ever be used. If not, the
    current (i,j) pair is abandoned and the next is
    considered (usually (i+i,j), incindentally (0,j+1)).
    If the Reduction Matrix says work needs to be done,
    it depends on the contents of the Reduction Matrix
    what happens next. If the Reduction Matrix points
    to a smaller Partial_PS value, that value is entered
    instead. If it contains a -2 or -3 (irreducible, or
    bow), the innermost loop is run. This loop runs over
    all k for which a bow i->k, i+1->k-1 exists.
    Before the loop starts, the partition sum is
    initialised at length j-1. The loop then adds all
    possible configurations to the partition sum. The
    result is the partition sum consisting of all
    possible configurations, except for those which have
    a start-end bow. Such configurations may or may not
    exist, depending on whether or not the first two
    nucleotides on the partial strand, match the last
    two nucleotides.
    Finally, if there were any start-end configurations,
    these configurations are added to Partial_PSbow.
    These will be used in other iterations.

    Returned value:
    The returned value is the sum of all possible
    configurations, hence it consists of Partial_PS, all
    configurations without the start-end bow, and
    Partial_PSbow, which (if nonzero) contains the
    configurations which had the first and last
    nucleotide connected to each other.
*/
// =====================================================

double partsum(short int * strand, int strlength, double ** expbdg, double** Partial_PS,
	double** Partial_PSbow, int** offset, int** kliststart, short int ** redmat)
{
    if(MINBOW >= strlength-2) return 1;

    int i,j,k;
    k = 0;
    int * ptr;
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    if(ptr[0]-i+2 > 3+MINBOW) Partial_PSbow[i][ptr[0]-i+2] = 1; // For future reference, a bow should come here
	}
    }

    for (j=MINBOW+3; j<strlength+1; j++) // j is the bowlength
    {
	for (i=0; i < (strlength+1 - j); i++) // i is the starting position of the i-j partition sum under consideration
	{
	    if(redmat[i][2*j] == -1) continue; // Partial_PS is initialized at 1
	    if(redmat[i][2*j+1] > 0) {Partial_PS[i][j] = Partial_PS[redmat[i][2*j]][redmat[i][2*j+1]]+Partial_PSbow[redmat[i][2*j]][redmat[i][2*j+1]]; continue;}
	    Partial_PS[i][j] = Partial_PS[i+1][j-1];	// Start with the No-Bow case, there is NO i->j bow.
	    Partial_PS[i][j] += Partial_PSbow[i+1][j-1]; // Now, we DO want to allow this bow in the partition sum
	    for(ptr=kliststart[i]; ptr<offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	    {
		k = ptr[0]+2-i; // +1 because the k is the first of the pair (in the list of k's).
				// We want an i -> k and i+1 -> k-1 arc.
				// But also +1 because k is the number
				// of positions, k=i_end-i_start+1.
		if(k > j-1) break; // Too far, bow exceeds partsum. Recall that partsum ends at position j-1.
		if(k < MINBOW+4) continue; // Too small bow, perhaps next bow is better
		Partial_PS[i][j] += Partial_PSbow[i][k]*(Partial_PS[i+k][j-k] + Partial_PSbow[i+k][j-k]);
	    }
	    if(Partial_PSbow[i][j] == 1)
	    {
		Partial_PSbow[i][j] = expbdg[strand[i]][strand[i+j-2]]*
		    (Partial_PSbow[i+1][j-2] + Partial_PS[i+2][j-4]);
	    }
	}
    }
    double partsum = Partial_PS[0][strlength] + Partial_PSbow[0][strlength];

  return partsum;
    
} // End of iterative algo with klist and redmat

// =====================================================
//   PARTITION FUNCTION (from Partsum_NN.cpp)

/*  The Partition Function will use the known values
    for the pair binding energies to determine the
    total partition sum for a given configuration.
    To determine the Partition Sum, a double sum has to
    be evaluated.
    Function variables:
    * strand: A strand of numbers 0-7 to represent RNA
    * strlength: The length of the Dyn. All. strand
    * expbdg: binding energy values
    * Partial_PS: array for storage of results
    * Partial_PSbow: array for storage of results with start-end arc

    Characteristics:
    This is the most intuitive approach. All partition
    sums are calculated, using old results.

    Method summary:
    This function consists of a triple loop. The
    outermost loop runs over the length j of the partial
    strand. The second runs over the position i. Then,
    for each partial partition sum p(i,j),
    initialization is done: p(i,j) = p(i,j-1) +
    pbow(i,j-1). All possible arcs connecting to the
    j-th nucleotide are then found and added to the
    total. In this manner, all partial partition sums
    are computed in an order-N^3 algorithm.

    Returned value:
    The returned value is the sum of all possible
    configurations, hence it consists of Partial_PS, all
    configurations without the start-end bow, and
    Partial_PSbow, which (if nonzero) contains the
    configurations which had the first and last
    nucleotide connected to each other.
*/
// =====================================================

double partsum(short int * strand, int strlength, double ** expbdg, double ** Partial_PS, double ** Partial_PSbow)
{
    if(MINBOW >= strlength-2) return 1; // There can be no possible connection. But
    		// ... if we continue, terrible things happen, so return 1 right now.
    int i, j, k;  // defining the necessary variables for the triple loop
    double betadeltag; // To keep notation short in calculations
    double prebow; // The partition sum ahead of the bow, which may have a bow i->k-1
    double innerbow; // The partition sum inside AND including the bow i+k->i+j
  
    for (j=MINBOW+3; j<strlength; j++) // j is the bowlength
    {
	for (i=0; i < (strlength - j); i++) // i is the starting position of the i-j partition sum under consideration
	{
	    Partial_PS[i][j] = Partial_PS[i][j-1];	// Start with the No-Bow case, there is NO i->j bow.
	    Partial_PS[i][j] += Partial_PSbow[i][j-1]; // Now, we DO want to allow this bow in the partition sum
	    for (k=0; k<j-2-MINBOW; k++)		// Consider a bow i+k->i+j and i+k+1->i+j-1
	    {
// To fit this on a page, the indent was shifted to the left!

// First, check if the bow is favorable, if nucleotides are not complementary, quit current loop.
// To do this, consider only those combinations whose indices sum up to 3 or 7 or 11.
// Since dAdT = 0+3, dCdG = 1+2, dArU = 0+7, dCrG = 1+6, rArU = 4+7, et cetera,
// and no illegal combination of two elements of the
// set {0,1,2,3,4,5,6,7} gives 3 modulo 4. (Solutions are 0+3,1+2,0+7,1+6,2+5,3+4,4+7,5+6
// which correspond to dAdT,dCdG,dArU,dCrG,dGrC,dTrA,rArU,rCrG. Thus, we have a one-to-one
// mapping between pairs of numbers <8 and NN pairs, designed such that every complementary
// pair sums up to 3 mod 4, while every noncomplementary pair sums up to something else.
		
// Don't do anything unless there is a bow
if( ((strand[i+k] + strand[i+j]) % 4) != 3 || ((strand[i+k+1] + strand[i+j-1]) % 4) != 3)
    continue; // some things don't match
if( (strand[i+k] + strand[i+j]) > 14 || strand[i+k+1] + strand[i+j-1] > 14)
    continue; // we're having X's here, forbidden

// Calculate the value of the exponent for this bow
betadeltag = expbdg[strand[i+k]][strand[i+j-1]]; // NOTE HOW THIS ALGORITHM USES A DIFFERENT NOTATION, such that [i][j] has j+1 nucleotides.

// Calculate inner bow and prebow
// Prebow consists of the partition sum (bow or no-bow i->i+k-1) from the start (i) to the bow (i+k-1)
if(k>3) prebow = Partial_PS[i][k-1] + Partial_PSbow[i][k-1]; // The TOTAL partition sum for i up to i+k-1
else prebow = 1;
// Inner bow consists of (Bow energy) x (Partition sum of i+k+2->i+j-2 with NO bow between the ends)
// 		    +	 (Bow energy) x (Partition sum of i+k+1->i+j-1 with FORCED bow between the ends)
if(k<j-3) innerbow = betadeltag*Partial_PS[i+k+2][j-k-4];
else innerbow = 1;
innerbow += betadeltag*Partial_PSbow[i+k+1][j-k-2];
// in the MINBOW=0 case, the "if" statement is necessary to prevent
// segmentation faults. Else, this will not occur.

Partial_PSbow[i+k][j-k] = innerbow;
if(k>0) Partial_PS[i][j] += innerbow*prebow;
	    }
	}
    }
    double partsum = Partial_PS[0][strlength-1] + Partial_PSbow[0][strlength-1];

  return partsum;
}

// =====================================================
//   end PARTITION FUNCTION
// =====================================================

// =====================================================
//  runTest(void): the Algorithm Testing function
/*
    Characteristics:
    This function creates tables required by the more
    advanced algorithms, and then runs them repeatedly
    for various strengths to gather representative data
    for the computation time required for each
    algorithm.

    Method summary:
    The function loops over a number of different
    strands. For each strand, the tables are created
    multiple times for an accurate estimate of the time
    required to do so.  When this is done, the
    algorithms are run many times. There are also
    (slightly obsolete) shadow versions of these
    algorithms, which have a number of counters to count
    how often basic operations (multiplications, array
    lookups, array writes, function calls, et cetera)
    are performed. These are run once.
    When the loop over all strands has finised, the time
    each algorithm required for the list of strands, is
    finally printed to screen in Mathematica-list
    format.

    Additional notes:
    The list of binding energies is a dummy list whose
    values are only marginally correct. However, for the
    purpose of testing algorithm efficiency, the precise
    values are of little importance.
*/
// =====================================================

void addDNA(string * &strandl, int i, string newstr)
{
    string * strandl1 = new string[i+1];
    for(int j = 0; j<i; j++) strandl1[j] = strandl[j];
    strandl1[i] = newstr;
    delete [] strandl;
    strandl = strandl1;
}

void runTest(void)
{
    int i,j,k;
    double * bindEnergarr;
    double * bindEnerg[8];
    bindEnergarr = new double[64];
    for(i = 0; i<8; i++)
	bindEnerg[i] = bindEnergarr + 8*i;

const int REPBASE = 50000000; // The number of repetitions for each strand. Ideally, each strand should take a few seconds to complete, for timings. For call counts, this may be very low.
const int NUMSTR = 32; // The number of different strings at each length
const int MAXLEN = 801;
const int NUM_RUNS = 1; // The number of times everything is done. Increase to get smoother results (repeat calculations)
int lengthnum = 0; // The number of different lengths
int strnum = 0; // The number of different strands
string * strandl; 
strandl = new string[1];
for(i = 4; i<MAXLEN; i++)
{
    if(i<11 && i % 2 != 0) continue; // Don't do this i
    if(i>10 && i<21 && i % 4 != 0) continue;
    if(i>20 && i<41 && i % 8 != 0) continue;
    if(i>40 && i<161 && i % 16 != 0) continue;
    if(i>160 && i<401 && i % 32 != 0) continue;
    if(i>400 && i<10001 && i % 42 != 0) continue;
    cout << "Making " << NUMSTR << " strings of length " << i << "...\n";

    lengthnum++;
    for(j = 0; j<NUMSTR; j++)
    {
	string str = mkDNA(i);
	addDNA(strandl, strnum, str);
	strnum++;
    }
}
int runs = 0; // The number of times the entire program is run
double * ticks = new double[strnum*COUNTNUM]; // Store run time of used strands (COUNTNUM is the number of separate counters)
int * calls = new int[strnum*9]; // Store number of function calls
for(i = 0; i<strnum*COUNTNUM; i++) ticks[i] = 0;
for(i = 0; i<strnum*9; i++) calls[i] = 0;
int * strlengtharray = new int[strnum]; // Store lengths of used strands


// Initialize bindEnerg array for T=37`C

// Values correspond to the parameters via a mapping dAdC/dTdG -> [0][2], dAdG/rUrC -> [0][5], etc. (note also that [i][j] = [j][i] in this mapping)
// (So A=0, C=1, ..., U=7, and we take the first and last character of aWaX/bYbZ where a,b in {d,r} and W,X,Y,Z in {A,C,G,T/U} )
// Last update: January 20, 2009.
    
bindEnerg[0][0] = 0.88;
bindEnerg[0][1] = 1.28;
bindEnerg[0][2] = 1.44;
bindEnerg[0][3] = 1.00;
bindEnerg[0][4] = 0.9;
bindEnerg[0][5] = 0.9;
bindEnerg[0][6] = 1.1;
bindEnerg[0][7] = 0.2;

bindEnerg[1][0] = 1.28;
bindEnerg[1][1] = 2.17;
bindEnerg[1][2] = 1.84;
bindEnerg[1][3] = 1.45;
bindEnerg[1][4] = 1.8;
bindEnerg[1][5] = 1.7;
bindEnerg[1][6] = 2.9;
bindEnerg[1][7] = 1.6;

bindEnerg[2][0] = 1.44;
bindEnerg[2][1] = 1.84;
bindEnerg[2][2] = 2.24;
bindEnerg[2][3] = 1.30;
bindEnerg[2][4] = 2.1;
bindEnerg[2][5] = 2.1;
bindEnerg[2][6] = 2.7;
bindEnerg[2][7] = 1.5;

bindEnerg[3][0] = 1.00;
bindEnerg[3][1] = 1.45;
bindEnerg[3][2] = 1.30;
bindEnerg[3][3] = 0.58;
bindEnerg[3][4] = 1.0;
bindEnerg[3][5] = 0.9;
bindEnerg[3][6] = 1.3;
bindEnerg[3][7] = 0.6;

bindEnerg[4][0] = 0.9;
bindEnerg[4][1] = 1.8;
bindEnerg[4][2] = 2.1;
bindEnerg[4][3] = 1.0;
bindEnerg[4][4] = 1.10;
bindEnerg[4][5] = 2.08;
bindEnerg[4][6] = 2.24;
bindEnerg[4][7] = 0.93;

bindEnerg[5][0] = 0.9;
bindEnerg[5][1] = 1.7;
bindEnerg[5][2] = 2.1;
bindEnerg[5][3] = 0.9;
bindEnerg[5][4] = 2.08;
bindEnerg[5][5] = 2.36;
bindEnerg[5][6] = 3.26;
bindEnerg[5][7] = 2.11;

bindEnerg[6][0] = 1.1;
bindEnerg[6][1] = 2.9;
bindEnerg[6][2] = 2.7;
bindEnerg[6][3] = 1.3;
bindEnerg[6][4] = 2.24;
bindEnerg[6][5] = 3.26;
bindEnerg[6][6] = 3.42;
bindEnerg[6][7] = 2.35;

bindEnerg[7][0] = 0.2;
bindEnerg[7][1] = 1.6;
bindEnerg[7][2] = 1.5;
bindEnerg[7][3] = 0.6;
bindEnerg[7][4] = 0.93;
bindEnerg[7][5] = 2.11;
bindEnerg[7][6] = 2.35;
bindEnerg[7][7] = 1.33;

for(i = 0; i<8; i++)
    for(j = 0; j<8; j++)
	bindEnerg[i][j] = exp(beta*bindEnerg[i][j]);

for(int nstr = 0; nstr < strnum; nstr++)
{
    string chrstrand = strandl[nstr];
    int strlength = chrstrand.length();
    cout << "Doing partsum calculation... using string of length " << strlength;
    cout << ". This is run " << nstr+runs*strnum << " out of " << NUM_RUNS*strnum << " total." << "String is: " << chrstrand << endl;
    if(strlength < 1 || strlength > 100000) {cout << "NUMSTR was likely too large, strlength out of range! Aborting!\n"; break; } // If no string present, quit!!! 
    strlengtharray[nstr] = strlength;
    void translate(string, short int *);
    short int * strand;
    strand = new short int[strlength];
    translate(chrstrand, strand);

    int reps = static_cast<int>(REPBASE/(pow(strlength,3.0))); // Offset for number of repetitions. Calculation time is expected to follow approx. n^2.5 in most cases.
    if(reps < 10) reps = 10; // At least ten times, say
    double * matrarr; // Matrix for partition sums
    double ** matr = new double*[strlength]; // Actual pointers to rows
    double * matrbowarr; // Matrix for part. sums with start-end arc
    double ** matrbow = new double*[strlength]; // Actual pointers to rows
    matrarr = new double[(strlength+1)*strlength];
    matrbowarr = new double[(strlength+1)*strlength];
    for(i = 0; i<strlength; i++)
    {
	matr[i] = matrarr + i*(strlength+1);
	matrbow[i] = matrbowarr + i*(strlength+1);
    }

// ================ PART 1: CREATE LIST OF K's =============
    
    int * klist; // list of k's, sorted by species
    int ** offset; // aid creating an offset
    int kcount; // number of possible k's for all i
    int countpairs[4][4]; // Here, store the pairs encountered.
    			  // First index also defines DNA or RNA.

    offset = new int*[17];
    
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // initialize
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	    countpairs[strand[i]%4][strand[i+1]%4]++;
    kcount = 0;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    kcount += countpairs[i][j];
    klist = new int[kcount+1]; // Add one to name 'last'
    offset[0] = klist;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    offset[4*i+j+1] = offset[4*i+j] + countpairs[i][j]; // I'd have expected a sizeof(int) here, but apparently
    								// that was already taken into account. But I do not know why!
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // reset to 0
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	{
	    j = strand[i] % 4;
	    k = strand[i+1] % 4;
	    offset[4*j+k][countpairs[j][k]] = i;
	    countpairs[j][k]++; // Keep track of number of pairs found so far
	}

int incr = 0;
clock_t clstart, clfinish;
clstart = clock();
for(incr = 0; incr<reps*strlength*sqrt(strlength); incr++) // Only linear in N, I guess
{
    // First delete, then recreate
    delete [] matrarr;
    delete [] matrbowarr;
    delete [] offset;
    delete [] klist;
    matrarr = new double[(strlength+1)*strlength];
    matrbowarr = new double[(strlength+1)*strlength];
    for(i = 0; i<strlength; i++)
    {
	matr[i] = matrarr + i*(strlength+1);
	matrbow[i] = matrbowarr + i*(strlength+1);
    }

    offset = new int*[17];
    
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // initialize
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	    countpairs[strand[i]%4][strand[i+1]%4]++;
    kcount = 0;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    kcount += countpairs[i][j];
    klist = new int[kcount+1]; // Add one to name 'last'
    offset[0] = klist;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    offset[4*i+j+1] = offset[4*i+j] + countpairs[i][j]; // I'd have expected a sizeof(int) here, but apparently
    								// that was already taken into account. But I do not know why!
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // reset to 0
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	{
	    j = strand[i] % 4;
	    k = strand[i+1] % 4;
	    offset[4*j+k][countpairs[j][k]] = i;
	    countpairs[j][k]++; // Keep track of number of pairs found so far
	}
} // end repeating
clfinish = clock();
ticks[COUNTNUM*nstr + 5] += double(clfinish-clstart)/(reps*strlength*sqrt(strlength));
cout << "Create list of k's took               " << double(clfinish-clstart)/(reps*strlength*sqrt(strlength)) << " ticks.\n";

// =============== PART 2: CREATE LIST OF STARTING POINTS =====

    int ** kliststart;
    int * offsDNA1;
    int * offsDNA2;
    int * last = offset[16]; // Pointer to end of klist array
    last[0] = strlength;
    kliststart = new int*[strlength-1]; // Now, kliststart lists addresses for each i.
    				      // The address is in the klist array, which is
    				      // also partitioned by the offset[] pointer list.
    				      // When used, offset should be combined with this
    				      // list to find the required list of k's.
    for(i = 0; i<3; i++)
	for(j = 0; j<3-i; j++) // Not the 4x4 square, but only the
	    		       // left-upper triangle without antidiagonal
	{
	    offsDNA1 = offset[4*i+j]; // Define offsets for pairs, first: DNA(i,j) or RNA(i,j)
	    offsDNA2 = offset[4*(3-j)+3-i]; // DNA complementary to either DNA(i,j) or RNA(i,j)
	    // Only keep going while there are pairs to be found
	    if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
	    if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    while(offsDNA1 < last || offsDNA2 < last) // When no more matches, point to 'last'!
	    {
		// Write into kliststart.
		if(offsDNA1[0] < offsDNA2[0])
		{
		    kliststart[offsDNA1[0]]=offsDNA2;
		    offsDNA1++;
		}
		else
		{
		    kliststart[offsDNA2[0]]=offsDNA1;
		    offsDNA2++;
		}
		// Disable offset when we've reached the end of the list of pairs
		if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
		if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    }
	}
    for(i = 0; i<4; i++) // Now do the antidiagonal we omitted here.
			 // Those are self-complementary pairs.
	{
	    offsDNA1 = offset[4*i+(3-i)]; // Selfcomplementary, so i+j=3, j = 3-i
	    while(offsDNA1 < offset[4*i+(3-i)+1]-1) // Not the last one!
	    {
		kliststart[offsDNA1[0]] = ++offsDNA1; // Point to next pair in the list.
	    }
	    if(offset[4*i+(3-i)] < offset[4*i+(3-i)+1])
	    {
		kliststart[offsDNA1[0]] = last; // Last one points to 'last'! But only if it EXISTS!
	    }
	}

    for(i = 0; i<strlength-1; i++)
	if(strand[i] > 8 || strand[i+1] > 8)
	{
	    kliststart[i] = last;
	}
clstart = clock(); // Now see how much time this cost
for(incr = 0; incr<0.2*reps*strlength*sqrt(strlength); incr++)
{
    // Delete, then recreate
    delete [] kliststart;
    last[0] = strlength;
    kliststart = new int*[strlength-1]; // Now, kliststart lists addresses for each i.
    				      // The address is in the klist array, which is
    				      // also partitioned by the offset[] pointer list.
    				      // When used, offset should be combined with this
    				      // list to find the required list of k's.
    for(i = 0; i<3; i++)
	for(j = 0; j<3-i; j++) // Not the 4x4 square, but only the
	    		       // left-upper triangle without antidiagonal
	{
	    offsDNA1 = offset[4*i+j]; // Define offsets for pairs, first: DNA(i,j) or RNA(i,j)
	    offsDNA2 = offset[4*(3-j)+3-i]; // DNA complementary to either DNA(i,j) or RNA(i,j)
	    // Only keep going while there are pairs to be found
	    if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
	    if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    while(offsDNA1 < last || offsDNA2 < last) // When no more matches, point to 'last'!
	    {
		// Write into kliststart.
		if(offsDNA1[0] < offsDNA2[0])
		{
		    kliststart[offsDNA1[0]]=offsDNA2;
		    offsDNA1++;
		}
		else
		{
		    kliststart[offsDNA2[0]]=offsDNA1;
		    offsDNA2++;
		}
		// Disable offset when we've reached the end of the list of pairs
		if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
		if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    }
	}
    for(i = 0; i<4; i++) // Now do the antidiagonal we omitted here.
			 // Those are self-complementary pairs.
	{
	    offsDNA1 = offset[4*i+(3-i)]; // Selfcomplementary, so i+j=3, j = 3-i
	    while(offsDNA1 < offset[4*i+(3-i)+1]-1) // Not the last one!
	    {
		kliststart[offsDNA1[0]] = ++offsDNA1; // Point to next pair in the list.
	    }
	    if(offset[4*i+(3-i)] < offset[4*i+(3-i)+1])
	    {
		kliststart[offsDNA1[0]] = last; // Last one points to 'last'! But only if it EXISTS!
	    }
	}

    for(i = 0; i<strlength-1; i++)
	if(strand[i] > 8 || strand[i+1] > 8)
	{
	    kliststart[i] = last;
	}
} // end repeating
clfinish = clock();
ticks[COUNTNUM*nstr + 6] += double(clfinish-clstart)/(0.2*reps*strlength*sqrt(strlength));
cout << "Create list of starting points took   " << double(clfinish-clstart)/(0.2*reps*strlength*sqrt(strlength)) << " ticks.\n";

// Build matrbow array for Reduction matrix...
    int * ptr; // Pointer as loop variable (Loop through array)
// INITIALIZE PARTIAL RESULT ARRAYS
clstart = clock();
for(incr = 0; incr < 10*reps; incr++) 
{
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	    matr[i][j] = matrbow[i][j] = 0;
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    matrbow[i][ptr[0]-i+2] = 1;
	}
    }
} // end repeating
clfinish = clock();
ticks[COUNTNUM*nstr + 7] += double(clfinish-clstart)/(10*reps);
cout << "Initialize partial result arrays took " << double(clfinish-clstart)/(10*reps) << " ticks.\n";

// ========================== REDUCTION MATRIX ==========================
    
/* Reduction matrix procedure:
   We wish to fill a matrix (of pairs) with information on matrix irreducibility.
   The convention used is the following:
   -3 : There is a start-end bow to this partial partition sum.
   -2 : There is no start-end bow to this ppsum, but it is not reducible.
   -1 : There are no bows possible in this ppsum, its value is thus equal to 1.
   i,j (i,j>-1) : The ppsum is reducible and its value equals that of the irreducible
   		 ppsum of position i, length j.
   Note that, although the matrix consists of pairs, only in the last case, both entries
   contain useful information. In the other cases, these values are not initialized.
*/
    
// Creating a Reduction Matrix

    short int * redmatarr = new short int[2*strlength*(strlength+1)]; // Create in memory
    short int ** redmat = new short int*[strlength]; // List of row addresses
    for(i = 0; i<strlength; i++)
	redmat[i] = redmatarr + 2*i*(strlength+1);
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	    redmat[i][2*j] = redmat[i][2*j+1] = 0;
    for(i = 0; i<strlength; i++)
	redmat[i][0] = -1; // Write -1 into first column (corresponding to length 0)
    redmat[strlength-1][2] = -1; // Add one extra
    for(i = strlength-2; i>-1; i--) // Go top-down. Skip the line i=last elt. No bows anyways.
	for(j = 1; j<strlength-i+1; j++)
	{
	    if(matrbow[i][j] == 1) redmat[i][2*j] = -3; // There is a bow!
	    else if(redmat[i][2*j-2] < -1) // Anything connecting to first base?
	    {
		// Next 2 if statements: Anything connecting to end base?
		if(redmat[i+1][2*j-2] < -1) redmat[i][2*j] = -2; // Irreducible, since both parents are
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // If upper parent refers to some point on this diagonal
		    redmat[i][2*j] = -2; // Irreducible, both upper and lower ancestry contain a bow
					 // (i.e. both start and end are occupied by some bow)
		else if(redmat[i+1][2*j-2] == -1) // Upper parent no bows?
		{				  // Then refer to lower parent
		    redmat[i][2*j] = i;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] < i+j) // Referring to sub-diagonal point
		{
		    redmat[i][2*j] = i;		// We can refer to the lower parent, since
		    redmat[i][2*j+1] = j-1;	// some upper parent is doing that, too.
		}    				// (So there is no bow connecting to the end base.)
		else
		{
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] == -1) // Lower parent no bows?
	    {
		if(redmat[i+1][2*j-2] == -1) redmat[i][2*j] = -1; // Upper parent no bows, either?
		else if(redmat[i+1][2*j-2] < -1) // Upper parent irreducible?
		{
		    redmat[i][2*j] = i+1; // Then refer to upper parent
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // Upper parent refers somewhere?
		{
		    redmat[i][2*j] = redmat[i+1][2*j-2];
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];
		}
		else // Note that as the lower parent had no bows anywhere, if the upper parent refers
		{    // anywhere, it is required to contain a bow connecting to the end.
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] > i) // Referring to something ABOVE this line? (If not, BAD OMEN!)
	    {
		if(redmat[i+1][2*j-2] < -1) // Apparently some irreducible thing as upper parent
		{
		    redmat[i][2*j] = i+1;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-2] == i+j) // Diagonal contains irr?
		{ // Then copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
		else
		{ // Copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
	    }
	    else if(redmat[i][2*j-2] == i) // Referring to something ON this line?
	    {
		// Some irreducible thing as upper parent, or in upper ancestry (bow connects to end)
		if(redmat[i+1][2*j-2] < -1 || redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) redmat[i][2*j] = -2;
		else // Apparently, nothing irreducible in upper ancestry
		{
		    redmat[i][2*j] = redmat[i][2*j-2];
		    redmat[i][2*j+1] = redmat[i][2*j-1];
		}
	    }
	    else // Note that as the lower parent had no bows anywhere, if the upper parent refers
	    {    // anywhere, it is required to contain a bow connecting to the end.
		cout << "LOWER parent error:\n";
		cout << "Matrix entry for ["<<i<<"][2*"<<j-1<<"] was ("<<redmat[i][2*j-2];
		cout << ","<<redmat[i][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		cout << "Aborting.\n"; exit(1);
	    }
	} // End j loop (length) (i loop (position) ends here, too)

// Do it again, and see how much time it takes to do this

int redmatreps = 5*reps;
clstart = clock();
for(incr = 0; incr<redmatreps; incr++)
{
    delete [] redmatarr;
    redmatarr = new short int[2*strlength*(strlength+1)]; // Create in memory
    for(i = 0; i<strlength; i++)
	redmat[i] = redmatarr + 2*i*(strlength+1);
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	    redmat[i][2*j] = redmat[i][2*j+1] = 0;
    for(i = 0; i<strlength; i++)
	redmat[i][0] = -1; // Write -1 into first column (corresponding to length 0)
    redmat[strlength-1][2] = -1; // Add one extra
    for(i = strlength-2; i>-1; i--) // Go top-down. Skip the line i=last elt. No bows anyways.
	for(j = 1; j<strlength-i+1; j++)
	{
	    if(matrbow[i][j] == 1) redmat[i][2*j] = -3; // There is a bow!
	    else if(redmat[i][2*j-2] < -1) // Anything connecting to first base?
	    {
		// Next 2 if statements: Anything connecting to end base?
		if(redmat[i+1][2*j-2] < -1) redmat[i][2*j] = -2; // Irreducible, since both parents are
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // If upper parent refers to some point on this diagonal
		    redmat[i][2*j] = -2; // Irreducible, both upper and lower ancestry contain a bow
					 // (i.e. both start and end are occupied by some bow)
		else if(redmat[i+1][2*j-2] == -1) // Upper parent no bows?
		{				  // Then refer to lower parent
		    redmat[i][2*j] = i;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] < i+j) // Referring to sub-diagonal point
		{
		    redmat[i][2*j] = i;		// We can refer to the lower parent, since
		    redmat[i][2*j+1] = j-1;	// some upper parent is doing that, too.
		}    				// (So there is no bow connecting to the end base.)
		else
		{
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] == -1) // Lower parent no bows?
	    {
		if(redmat[i+1][2*j-2] == -1) redmat[i][2*j] = -1; // Upper parent no bows, either?
		else if(redmat[i+1][2*j-2] < -1) // Upper parent irreducible?
		{
		    redmat[i][2*j] = i+1; // Then refer to upper parent
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // Upper parent refers somewhere?
		{
		    redmat[i][2*j] = redmat[i+1][2*j-2];
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];
		}
		else // Note that as the lower parent had no bows anywhere, if the upper parent refers
		{    // anywhere, it is required to contain a bow connecting to the end.
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] > i) // Referring to something ABOVE this line? (If not, BAD OMEN!)
	    {
		if(redmat[i+1][2*j-2] < -1) // Apparently some irreducible thing as upper parent
		{
		    redmat[i][2*j] = i+1;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-2] == i+j) // Diagonal contains irr?
		{ // Then copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
		else
		{ // Copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
	    }
	    else if(redmat[i][2*j-2] == i) // Referring to something ON this line?
	    {
		// Some irreducible thing as upper parent, or in upper ancestry (bow connects to end)
		if(redmat[i+1][2*j-2] < -1 || redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) redmat[i][2*j] = -2;
		else // Apparently, nothing irreducible in upper ancestry
		{
		    redmat[i][2*j] = redmat[i][2*j-2];
		    redmat[i][2*j+1] = redmat[i][2*j-1];
		}
	    }
	    else // Note that as the lower parent had no bows anywhere, if the upper parent refers
	    {    // anywhere, it is required to contain a bow connecting to the end.
		cout << "LOWER parent error:\n";
		cout << "Matrix entry for ["<<i<<"][2*"<<j-1<<"] was ("<<redmat[i][2*j-2];
		cout << ","<<redmat[i][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		cout << "Aborting.\n"; exit(1);
	    }
	} // End j loop (length) (i loop (position) ends here, too)
} // end repeating
clfinish = clock();
ticks[COUNTNUM*nstr + 8] += double(clfinish-clstart)/(1.0*redmatreps);
cout << "Building reduction matrix took        " << double(clfinish-clstart)/(1.0*redmatreps) << " ticks.\n";
// INITIALIZE PARTIAL RESULT ARRAYS
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	{
	    matr[i][j] = matrbow[i][j] = 0;
	    if (redmat[i][2*j] == -1) matr[i][j] = 1;
	}
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    matrbow[i][ptr[0]-i+2] = 1;
	}
    }
    double result = 0;	// to store the resulting partition sum for returning
    int count;
    clock_t begin, end;
    begin = clock();
    for(count = 0; count < reps; count++)
    {
	result = recpartsum(strand, strlength, bindEnerg, matr, matrbow, offset, kliststart, 1);
    }
    end = clock();
    ticks[COUNTNUM*nstr+1] += double(end-begin)/reps;
    cout << "Time taken for recursive algo:(klist)  " << double(end-begin)/reps << " ticks per run, result was " << result << endl;
    begin = clock();
    for(count = 0; count < reps; count++)
    {
	result = recpartsum(strand, strlength, bindEnerg, matr, matrbow, offset, kliststart, redmat, 1);
    }
    end = clock();
    ticks[COUNTNUM*nstr+3] += double(end-begin)/reps;
    cout << "Time taken for rec algo:     (redmat) " << double(end-begin)/reps << " ticks per run, result was " << result << endl;

    for(i=0; i<strlength; i++)
    {
	for(j=0; j<strlength+1; j++)
	{
	    matr[i][j] = 1; // All initialized to 1
	    matrbow[i][j] = 0; // initialize all to 0
	}
    }
    
    for (i=0; i<strlength; i++) matr[i][MINBOW+2] = 1.00000000000; // Initialize! (Necessary to use this algorithm)
    begin = clock();
    for(count = 0; count < reps; count++)
    {
	result = partsum(strand, strlength, bindEnerg, matr, matrbow);
    }
    end = clock();
    ticks[COUNTNUM*nstr+0] += double(end-begin)/(reps);
    cout << "Time taken for old algorithm:         " << double(end-begin)/(reps) << " ticks per run, result was " << result << endl;
    for(i=0; i<strlength; i++)
    {
	for(j=0; j<strlength+1; j++)
	{
	    matr[i][j] = 1; // All initialized to 1
	    matrbow[i][j] = 0; // initialize all to 0
	}
    }

    for (i=0; i<strlength; i++) matr[i][MINBOW+2] = 1.00000000000; // Initialize!
    begin = clock();
    for(count = 0; count < reps; count++)
    {
	result = partsum(strand, strlength, bindEnerg, matr, matrbow, offset, kliststart);
//	if(count % 1000 == 0) cout << "i = " << count << "...\n";
    }
    end = clock();
    ticks[COUNTNUM*nstr+2] += double(end-begin)/(reps);
    cout << "Time taken for old algorithm: (klist) " << double(end-begin)/(reps) << " ticks per run, result was " << result << endl;
    for(i=0; i<strlength; i++)
    {
	for(j=0; j<strlength+1; j++)
	{
	    matr[i][j] = 1; // All initialized to 1
	    matrbow[i][j] = 0; // initialize all to 0
	}
    }
    for (i=0; i<strlength; i++) matr[i][MINBOW+2] = 1.00000000000; // Initialize!
    begin = clock();
    for(count = 0; count < reps; count++)
    {
	result = partsum(strand, strlength, bindEnerg, matr, matrbow, offset, kliststart, redmat);
    }
    end = clock();
    ticks[COUNTNUM*nstr+4] += double(end-begin)/(reps);
    cout << "Time taken for old algorithm: (k+red) " << double(end-begin)/(reps) << " ticks per run, result was " << result << endl;


// Cleanup of allocated arrays starts here
    
    delete [] redmat;
    redmat = 0;
    delete [] redmatarr;
    redmatarr = 0;
    delete [] strand;
    strand = 0;
    delete [] matr;
    matr = 0;
    delete [] matrarr;
    matrarr = 0;
    delete [] matrbow;
    matrbow = 0;
    delete [] matrbowarr;
    matrbowarr = 0;
    delete [] offset;
    offset = 0;
    delete [] klist;
    klist = 0;
    delete [] kliststart;
    kliststart = 0;

    if(runs < NUM_RUNS-1 && nstr == strnum-1) // This run done for all strands? Then run again, if runs is still small enough.
    {
	runs++;
	nstr = -1;
    }
} // end nstr loop

for(i = 0; i<COUNTNUM*strnum; i++) ticks[i] *= 1/(double(NUM_RUNS));

double * averagelist = new double[lengthnum*COUNTNUM]; for(i = 0; i<lengthnum*COUNTNUM; i++) averagelist[i] = 0;
int pos=0;
int length;
int * lengthcounts = new int[lengthnum]; for(i = 0; i<lengthnum; i++) lengthcounts[i] = 0;
for(i = 0; i<lengthnum; i++)
{
    if(pos>strnum-1) {cout << "Position too large. Aborting average-ticks procedure.\n"; break;}
    length = strlengtharray[pos];
    strlengtharray[i] = length; // Use the old array to store the new lengths (the new list can never be longer than the old one)
    while(strlengtharray[pos] == length && pos<strnum-1)
    {
	for(j = 0; j<COUNTNUM; j++)
	{
	    averagelist[i*COUNTNUM+j] += ticks[pos*COUNTNUM+j]; // note that i =< pos
	}
	pos++;
	lengthcounts[i]++;
    }
}

for(i = 0; i<lengthnum; i++)
{
    for(j = 0; j<COUNTNUM; j++)
    {
        if(averagelist[i*COUNTNUM+j] < 0.0001) averagelist[i*COUNTNUM+j] = 0.0001;
	averagelist[i*COUNTNUM+j] /= lengthcounts[i];
    }
}


fstream outFile;

outFile.open("timings.txt", fstream::out);
if(!outFile) die("Failed to properly open file for output. Aborting.");

outFile << "Print length and time in list... Naive algorithm...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i  ];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Recursive algorithm with k-list...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+1];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Improved iterative algorithm with k-list...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+2];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Recursive algorithm with k-list and reduction matrix...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+3];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Improved iterative algorithm with k-list and reduction matrix...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+4];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... List of k's...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+5];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... List of starting points...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+6];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Matrix initialization...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+7];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Reduction matrix creation...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+8];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "\n\nNow printing summed lists...\n";
outFile << "Print length and time in list... Recursive algorithm with k-list...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+1]+averagelist[COUNTNUM*i+5]+averagelist[COUNTNUM*i+6];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Improved iterative algorithm with k-list...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+2]+averagelist[COUNTNUM*i+5]+averagelist[COUNTNUM*i+6];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Recursive algorithm with k-list and reduction matrix...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+3]+averagelist[COUNTNUM*i+5]+averagelist[COUNTNUM*i+6]+averagelist[COUNTNUM*i+8];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Print length and time in list... Improved iterative algorithm with k-list and reduction matrix...\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+4]+averagelist[COUNTNUM*i+5]+averagelist[COUNTNUM*i+6]+averagelist[COUNTNUM*i+8];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}

cout << "Everything written to file. Closing.\n";
outFile.close();

delete [] ticks;
delete [] calls;
delete [] strlengtharray;
delete [] averagelist;
delete [] lengthcounts;
}

// Main function.
int main(void)
{
    cout << "Algorithm testing program starting...\n";
    runTest();
    cout << "Finished calculations.\n";
    return 1;
}
