#include<iostream>
#include<fstream>
#include<iomanip>
#include<string>
#include<cstring>
#include<cmath>
#include"globvar.h" // some variables and function declarations
#include"rng.h"
#include<ctime>

using namespace std;

cRNG seed(7654321);

double r(void)
{
    return seed.Randf();
}

string mkDNA(int length)
{
    double temp;
    string str = "A"; // first base doesn't matter, by symmetry...
    for(int i = 0; i<length-1; i++)
    {
	temp = r();
	if(temp < 0.25) str.append("A");
	else if(temp < 0.5) str.append("C");
	else if(temp < 0.75) str.append("G");
	else str.append("T");
    }

    // Use the following code to create self-symmetric strands (i.e., strands that are complementary to itself as a whole)
   /* 
    str.append("X");
    for(int i = length-1; i>=0; i--)
    {
	if(str.at(i) == 'A') str.append("T");
	else if(str.at(i) == 'C') str.append("G");
	else if(str.at(i) == 'G') str.append("C");
	else str.append("A");
    }*/
    return str;
}


#define die(s) {printf(s); exit(1);} // Abort program, displaying the message (s)... (Use for fatal errors)
#define COUNTNUM 9 // The number of counters in the runTest function.


// =====================================================
//   TRANSLATE FUNCTION (from Partsum_NN.cpp)

/*	The translate(DNAstrand) function will change
	the chars in the strand to the values 0,1,2,3
	meaning A,C,G,T for DNA, and 4,5,6,7 meaning
	A,C,G,U for RNA.
	This will allow for a quicker calculation of the
	partition sum. These are dynamically allocated,
	to allow any strand length. Possibly, the memory
	cost can be cut down further by replacing short
	ints by something smaller, as I only need the
	integers 0,1,2,3,4,5,6 and 7. However, it should
	allow enumeration, to take some element from the
	binding energy matrix. Perhaps the "enum"
	statement solves this, but at the time of
	writing, this improvement seems hardly relevant.
	Moreover, the memory cost is only of order $N$,
	so these considerations do not seem very
	relevant.
*/
// =====================================================

void translate(string strand, short int * numstr)
{
    int i;
    int strlength = strand.length();

    for(i=0;i<strlength;i++)
    {
	numstr[i] = 8;

	if(strand.at(i) == 'A') numstr[i] = 0;
	if(strand.at(i) == 'C') numstr[i] = 1;
	if(strand.at(i) == 'G') numstr[i] = 2;
	if(strand.at(i) == 'T') numstr[i] = 3;
	if(strand.at(i) == 'B') numstr[i] = 4;
	if(strand.at(i) == 'D') numstr[i] = 5;
	if(strand.at(i) == 'H') numstr[i] = 6;
	if(strand.at(i) == 'U') numstr[i] = 7;
	if(strand.at(i) == 'X') numstr[i] = 15;

	if(numstr[i] == 8)
	{
	    cout << "FATAL ERROR!\n Invalid character '"<< strand.at(i) << "' encountered!\n\n";
	    exit(1);
	}
    }
}
// =====================================================
//   end TRANSLATE FUNCTION
// =====================================================

// =====================================================
//  runTest(void): the Algorithm Testing function
/*
    Characteristics:
    This function creates tables required by the more
    advanced algorithms, and then runs them repeatedly
    for various strengths to gather representative data
    for the computation time required for each
    algorithm.

    Method summary:
    The function loops over a number of different
    strands. For each strand, the tables are created
    multiple times for an accurate estimate of the time
    required to do so.  When this is done, the
    algorithms are run many times. There are also
    (slightly obsolete) shadow versions of these
    algorithms, which have a number of counters to count
    how often basic operations (multiplications, array
    lookups, array writes, function calls, et cetera)
    are performed. These are run once.
    When the loop over all strands has finised, the time
    each algorithm required for the list of strands, is
    finally printed to screen in Mathematica-list
    format.

    Additional notes:
    The list of binding energies is a dummy list whose
    values are only marginally correct. However, for the
    purpose of testing algorithm efficiency, the precise
    values are of little importance.
*/
// =====================================================

void addDNA(string * &strandl, int i, string newstr)
{
    string * strandl1 = new string[i+1];
    for(int j = 0; j<i; j++) strandl1[j] = strandl[j];
    strandl1[i] = newstr;
    delete [] strandl;
    strandl = strandl1;
}

void runTest(void)
{
    int i,j,k;
    double * bindEnergarr;
    double * bindEnerg[8];
    bindEnergarr = new double[64];
    for(i = 0; i<8; i++)
	bindEnerg[i] = bindEnergarr + 8*i;

const int NUMSTR = 160; // The number of different strings at each length
const int MAXLEN = 801;
int lengthnum = 0; // The number of different lengths
int strnum = 0; // The number of different strands
string * strandl; 
strandl = new string[1];
for(i = 4; i<MAXLEN; i++)
{
    if(i<11 && i % 2 != 0) continue; // Don't do this i
    if(i>10 && i<21 && i % 4 != 0) continue;
    if(i>20 && i<41 && i % 8 != 0) continue;
    if(i>40 && i<161 && i % 16 != 0) continue;
    if(i>160 && i<401 && i % 32 != 0) continue;
    if(i>400 && i<10001 && i % 42 != 0) continue;
    cout << "Making " << NUMSTR << " strings of length " << i << "...\n";

    lengthnum++;
    for(j = 0; j<NUMSTR; j++)
    {
	string str = mkDNA(i);
	addDNA(strandl, strnum, str);
	strnum++;
    }
}
int * calls = new int[strnum*9]; // Store number of function calls
for(i = 0; i<strnum*9; i++) calls[i] = 0;
int * strlengtharray = new int[strnum]; // Store lengths of used strands


// Initialize bindEnerg array for T=37`C

// Values correspond to the parameters via a mapping dAdC/dTdG -> [0][2], dAdG/rUrC -> [0][5], etc. (note also that [i][j] = [j][i] in this mapping)
// (So A=0, C=1, ..., U=7, and we take the first and last character of aWaX/bYbZ where a,b in {d,r} and W,X,Y,Z in {A,C,G,T/U} )
// Last update: January 20, 2009.
    
bindEnerg[0][0] = 0.88;
bindEnerg[0][1] = 1.28;
bindEnerg[0][2] = 1.44;
bindEnerg[0][3] = 1.00;
bindEnerg[0][4] = 0.9;
bindEnerg[0][5] = 0.9;
bindEnerg[0][6] = 1.1;
bindEnerg[0][7] = 0.2;

bindEnerg[1][0] = 1.28;
bindEnerg[1][1] = 2.17;
bindEnerg[1][2] = 1.84;
bindEnerg[1][3] = 1.45;
bindEnerg[1][4] = 1.8;
bindEnerg[1][5] = 1.7;
bindEnerg[1][6] = 2.9;
bindEnerg[1][7] = 1.6;

bindEnerg[2][0] = 1.44;
bindEnerg[2][1] = 1.84;
bindEnerg[2][2] = 2.24;
bindEnerg[2][3] = 1.30;
bindEnerg[2][4] = 2.1;
bindEnerg[2][5] = 2.1;
bindEnerg[2][6] = 2.7;
bindEnerg[2][7] = 1.5;

bindEnerg[3][0] = 1.00;
bindEnerg[3][1] = 1.45;
bindEnerg[3][2] = 1.30;
bindEnerg[3][3] = 0.58;
bindEnerg[3][4] = 1.0;
bindEnerg[3][5] = 0.9;
bindEnerg[3][6] = 1.3;
bindEnerg[3][7] = 0.6;

bindEnerg[4][0] = 0.9;
bindEnerg[4][1] = 1.8;
bindEnerg[4][2] = 2.1;
bindEnerg[4][3] = 1.0;
bindEnerg[4][4] = 1.10;
bindEnerg[4][5] = 2.08;
bindEnerg[4][6] = 2.24;
bindEnerg[4][7] = 0.93;

bindEnerg[5][0] = 0.9;
bindEnerg[5][1] = 1.7;
bindEnerg[5][2] = 2.1;
bindEnerg[5][3] = 0.9;
bindEnerg[5][4] = 2.08;
bindEnerg[5][5] = 2.36;
bindEnerg[5][6] = 3.26;
bindEnerg[5][7] = 2.11;

bindEnerg[6][0] = 1.1;
bindEnerg[6][1] = 2.9;
bindEnerg[6][2] = 2.7;
bindEnerg[6][3] = 1.3;
bindEnerg[6][4] = 2.24;
bindEnerg[6][5] = 3.26;
bindEnerg[6][6] = 3.42;
bindEnerg[6][7] = 2.35;

bindEnerg[7][0] = 0.2;
bindEnerg[7][1] = 1.6;
bindEnerg[7][2] = 1.5;
bindEnerg[7][3] = 0.6;
bindEnerg[7][4] = 0.93;
bindEnerg[7][5] = 2.11;
bindEnerg[7][6] = 2.35;
bindEnerg[7][7] = 1.33;

for(i = 0; i<8; i++)
    for(j = 0; j<8; j++)
	bindEnerg[i][j] = exp(beta*bindEnerg[i][j]);

for(int nstr = 0; nstr < strnum; nstr++)
{
    string chrstrand = strandl[nstr];
    int strlength = chrstrand.length();
    cout << "Doing partsum calculation... using string of length " << strlength;
    cout << ". This is run " << nstr << " out of " << strnum << " total." << "String is: " << chrstrand << endl;
    if(strlength < 1 || strlength > 100000) {cout << "NUMSTR was likely too large, strlength out of range! Aborting!\n"; break; } // If no string present, quit!!! 
    strlengtharray[nstr] = strlength;
    short int * strand;
    strand = new short int[strlength];
    translate(chrstrand, strand);

    double * matrarr; // Matrix for partition sums
    double ** matr = new double*[strlength]; // Actual pointers to rows
    double * matrbowarr; // Matrix for part. sums with start-end arc
    double ** matrbow = new double*[strlength]; // Actual pointers to rows
    matrarr = new double[(strlength+1)*strlength];
    matrbowarr = new double[(strlength+1)*strlength];
    for(i = 0; i<strlength; i++)
    {
	matr[i] = matrarr + i*(strlength+1);
	matrbow[i] = matrbowarr + i*(strlength+1);
    }

// ================ PART 1: CREATE LIST OF K's =============
    
    int * klist; // list of k's, sorted by species
    int ** offset; // aid creating an offset
    int kcount; // number of possible k's for all i
    int countpairs[4][4]; // Here, store the pairs encountered.
    			  // First index also defines DNA or RNA.

    offset = new int*[17];
    
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // initialize
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	    countpairs[strand[i]%4][strand[i+1]%4]++;
    kcount = 0;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    kcount += countpairs[i][j];
    klist = new int[kcount+1]; // Add one to name 'last'
    offset[0] = klist;
    for(i = 0; i<4; i++)
	for(j = 0; j<4; j++)
	    offset[4*i+j+1] = offset[4*i+j] + countpairs[i][j]; // I'd have expected a sizeof(int) here, but apparently
    								// that was already taken into account. But I do not know why!
    for(i = 0; i<4; i++)
       for(j = 0; j<4; j++)
	   countpairs[i][j] = 0; // reset to 0
    for(i = 0; i<strlength-1; i++)
	if(strand[i] < 8 && strand[i+1] < 8)
	{
	    j = strand[i] % 4;
	    k = strand[i+1] % 4;
	    offset[4*j+k][countpairs[j][k]] = i;
	    countpairs[j][k]++; // Keep track of number of pairs found so far
	}

// =============== PART 2: CREATE LIST OF STARTING POINTS =====

    int ** kliststart;
    int * offsDNA1;
    int * offsDNA2;
    int * last = offset[16]; // Pointer to end of klist array
    last[0] = strlength;
    kliststart = new int*[strlength-1]; // Now, kliststart lists addresses for each i.
    				      // The address is in the klist array, which is
    				      // also partitioned by the offset[] pointer list.
    				      // When used, offset should be combined with this
    				      // list to find the required list of k's.
    for(i = 0; i<3; i++)
	for(j = 0; j<3-i; j++) // Not the 4x4 square, but only the
	    		       // left-upper triangle without antidiagonal
	{
	    offsDNA1 = offset[4*i+j]; // Define offsets for pairs, first: DNA(i,j) or RNA(i,j)
	    offsDNA2 = offset[4*(3-j)+3-i]; // DNA complementary to either DNA(i,j) or RNA(i,j)
	    // Only keep going while there are pairs to be found
	    if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
	    if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    while(offsDNA1 < last || offsDNA2 < last) // When no more matches, point to 'last'!
	    {
		// Write into kliststart.
		if(offsDNA1[0] < offsDNA2[0])
		{
		    kliststart[offsDNA1[0]]=offsDNA2;
		    offsDNA1++;
		}
		else
		{
		    kliststart[offsDNA2[0]]=offsDNA1;
		    offsDNA2++;
		}
		// Disable offset when we've reached the end of the list of pairs
		if(offsDNA1 == offset[4*i+j+1]) offsDNA1 = last;
		if(offsDNA2 == offset[4*(3-j)+3-i+1]) offsDNA2 = last;
	    }
	}
    for(i = 0; i<4; i++) // Now do the antidiagonal we omitted here.
			 // Those are self-complementary pairs.
	{
	    offsDNA1 = offset[4*i+(3-i)]; // Selfcomplementary, so i+j=3, j = 3-i
	    while(offsDNA1 < offset[4*i+(3-i)+1]-1) // Not the last one!
	    {
		kliststart[offsDNA1[0]] = ++offsDNA1; // Point to next pair in the list.
	    }
	    if(offset[4*i+(3-i)] < offset[4*i+(3-i)+1])
	    {
		kliststart[offsDNA1[0]] = last; // Last one points to 'last'! But only if it EXISTS!
	    }
	}

    for(i = 0; i<strlength-1; i++)
	if(strand[i] > 8 || strand[i+1] > 8)
	{
	    kliststart[i] = last;
	}

// ========================== REDUCTION MATRIX ==========================
    
/* Reduction matrix procedure:
   We wish to fill a matrix (of pairs) with information on matrix irreducibility.
   The convention used is the following:
   -3 : There is a start-end bow to this partial partition sum.
   -2 : There is no start-end bow to this ppsum, but it is not reducible.
   -1 : There are no bows possible in this ppsum, its value is thus equal to 1.
   i,j (i,j>-1) : The ppsum is reducible and its value equals that of the irreducible
   		 ppsum of position i, length j.
   Note that, although the matrix consists of pairs, only in the last case, both entries
   contain useful information. In the other cases, these values are not initialized.
*/
    
// INITIALIZE PARTIAL RESULT ARRAYS (because we need that for creating the reduction matrix)
    int * ptr; // Pointer as loop variable (Loop through array)
    for(i = 0; i<strlength; i++)
        for(j = 0; j<strlength+1; j++)
            matr[i][j] = matrbow[i][j] = 0;
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
        if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
        for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
        {
            matrbow[i][ptr[0]-i+2] = 1;
        }
    }

// Creating a Reduction Matrix

    short int * redmatarr = new short int[2*strlength*(strlength+1)]; // Create in memory
    short int ** redmat = new short int*[strlength]; // List of row addresses
    for(i = 0; i<strlength; i++)
	redmat[i] = redmatarr + 2*i*(strlength+1);
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	    redmat[i][2*j] = redmat[i][2*j+1] = 0;
    for(i = 0; i<strlength; i++)
	redmat[i][0] = -1; // Write -1 into first column (corresponding to length 0)
    redmat[strlength-1][2] = -1; // Add one extra
    for(i = strlength-2; i>-1; i--) // Go top-down. Skip the line i=last elt. No bows anyways.
	for(j = 1; j<strlength-i+1; j++)
	{
	    if(matrbow[i][j] == 1) redmat[i][2*j] = -3; // There is a bow!
	    else if(redmat[i][2*j-2] < -1) // Anything connecting to first base?
	    {
		// Next 2 if statements: Anything connecting to end base?
		if(redmat[i+1][2*j-2] < -1) redmat[i][2*j] = -2; // Irreducible, since both parents are
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // If upper parent refers to some point on this diagonal
		    redmat[i][2*j] = -2; // Irreducible, both upper and lower ancestry contain a bow
					 // (i.e. both start and end are occupied by some bow)
		else if(redmat[i+1][2*j-2] == -1) // Upper parent no bows?
		{				  // Then refer to lower parent
		    redmat[i][2*j] = i;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] < i+j) // Referring to sub-diagonal point
		{
		    redmat[i][2*j] = i;		// We can refer to the lower parent, since
		    redmat[i][2*j+1] = j-1;	// some upper parent is doing that, too.
		}    				// (So there is no bow connecting to the end base.)
		else
		{
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] == -1) // Lower parent no bows?
	    {
		if(redmat[i+1][2*j-2] == -1) redmat[i][2*j] = -1; // Upper parent no bows, either?
		else if(redmat[i+1][2*j-2] < -1) // Upper parent irreducible?
		{
		    redmat[i][2*j] = i+1; // Then refer to upper parent
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) // Upper parent refers somewhere?
		{
		    redmat[i][2*j] = redmat[i+1][2*j-2];
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];
		}
		else // Note that as the lower parent had no bows anywhere, if the upper parent refers
		{    // anywhere, it is required to contain a bow connecting to the end.
		    cout << "Matrix entry for ["<<i+1<<"][2*"<<j-1<<"] was ("<<redmat[i+1][2*j-2];
		    cout << ","<<redmat[i+1][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		    cout << "Aborting.\n"; exit(1);
		}
	    }
	    else if(redmat[i][2*j-2] > i) // Referring to something ABOVE this line? (If not, BAD OMEN!)
	    {
		if(redmat[i+1][2*j-2] < -1) // Apparently some irreducible thing as upper parent
		{
		    redmat[i][2*j] = i+1;
		    redmat[i][2*j+1] = j-1;
		}
		else if(redmat[i+1][2*j-2] + redmat[i+1][2*j-2] == i+j) // Diagonal contains irr?
		{ // Then copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
		else
		{ // Copy the reference from upper parent
		    redmat[i][2*j] = redmat[i+1][2*j-2];	// We need the same irreducible ppsum as both		    
		    redmat[i][2*j+1] = redmat[i+1][2*j-1];	// parents, so we're free to choose either parent
		}
	    }
	    else if(redmat[i][2*j-2] == i) // Referring to something ON this line?
	    {
		// Some irreducible thing as upper parent, or in upper ancestry (bow connects to end)
		if(redmat[i+1][2*j-2] < -1 || redmat[i+1][2*j-2] + redmat[i+1][2*j-1] == i+j) redmat[i][2*j] = -2;
		else // Apparently, nothing irreducible in upper ancestry
		{
		    redmat[i][2*j] = redmat[i][2*j-2];
		    redmat[i][2*j+1] = redmat[i][2*j-1];
		}
	    }
	    else // Note that as the lower parent had no bows anywhere, if the upper parent refers
	    {    // anywhere, it is required to contain a bow connecting to the end.
		cout << "LOWER parent error:\n";
		cout << "Matrix entry for ["<<i<<"][2*"<<j-1<<"] was ("<<redmat[i][2*j-2];
		cout << ","<<redmat[i][2*j-1]<<") and apparently this didn't match any of the cases.\n";
		cout << "Aborting.\n"; exit(1);
	    }
	} // End j loop (length) (i loop (position) ends here, too)

// INITIALIZE PARTIAL RESULT ARRAYS (again)
    for(i = 0; i<strlength; i++)
	for(j = 0; j<strlength+1; j++)
	{
	    matr[i][j] = matrbow[i][j] = 0;
	    if (redmat[i][2*j] == -1) matr[i][j] = 1;
	}
    // For all i, list bows i->k but be careful to use only the right startlist.
    // This list might be empty, s.t. kliststart[i] points to 'last'=offset[16].
    for(i = 0; i<strlength-3; i++)
    {
	if(strand[i]>7 || strand[i+1]>7) continue; // X here, no offset needed
	for(ptr = kliststart[i]; ptr < offset[4*(3-strand[i+1]%4)+(3-strand[i]%4)+1]; ptr++)
	{
	    matrbow[i][ptr[0]-i+2] = 1;
	}
    }
    
    double result = 0;	// to store the resulting partition sum for returning
    double result2 = 0;
    int clist[20];
    for(i=0; i<20; i++) clist[i] = 0; // init
    result = recpartsum(strand, strlength, bindEnerg, matr, matrbow, offset, kliststart, redmat, 1, clist);
    calls[COUNTNUM*nstr  ] = clist[14];
    calls[COUNTNUM*nstr+1] = clist[12];
    calls[COUNTNUM*nstr+2] = clist[13];
    calls[COUNTNUM*nstr+3] = clist[4];
    calls[COUNTNUM*nstr+4] = clist[6];
    calls[COUNTNUM*nstr+5] = clist[9]+clist[10];
    
    for(i=0; i<strlength; i++)
    {
	for(j=0; j<strlength+1; j++)
	{
	    matr[i][j] = 1; // All initialized to 1
	    matrbow[i][j] = 0; // initialize all to 0
	}
    }
    for (i=0; i<strlength; i++) matr[i][MINBOW+2] = 1.00000000000; // Initialize!
    
    for(i=0; i<20; i++) clist[i] = 0; // init
    result2 = partsum(strand, strlength, bindEnerg, matr, matrbow, clist);
    calls[COUNTNUM*nstr+6] = clist[4];
    calls[COUNTNUM*nstr+7] = clist[6];
    calls[COUNTNUM*nstr+8] = clist[9]+clist[10];
//    cout << "Result was " << result << ", result2 was " << result2 << endl; (Enable if you like to see the results obtained)
    delete [] redmat;
    redmat = 0;
    delete [] redmatarr;
    redmatarr = 0;
    delete [] strand;
    strand = 0;
    delete [] matr;
    matr = 0;
    delete [] matrarr;
    matrarr = 0;
    delete [] matrbow;
    matrbow = 0;
    delete [] matrbowarr;
    matrbowarr = 0;
    delete [] offset;
    offset = 0;
    delete [] klist;
    klist = 0;
    delete [] kliststart;
    kliststart = 0;
} // end nstr loop

double * averagelist = new double[lengthnum*COUNTNUM]; for(i = 0; i<lengthnum*COUNTNUM; i++) averagelist[i] = 0;
int pos=0;
int length;
int * lengthcounts = new int[lengthnum]; for(i = 0; i<lengthnum; i++) lengthcounts[i] = 0;
for(i = 0; i<lengthnum; i++)
{
    if(pos>strnum-1) {cout << "Position too large. Aborting average-ticks procedure.\n"; break;}
    length = strlengtharray[pos];
    strlengtharray[i] = length; // Use the old array to store the new lengths (the new list can never be longer than the old one)
    while(strlengtharray[pos] == length && pos<strnum-1)
    {
	for(j = 0; j<COUNTNUM; j++)
	{
	    averagelist[i*COUNTNUM+j] += double(calls[pos*COUNTNUM+j]); // note that i =< pos
	}
	pos++;
	lengthcounts[i]++;
    }
}

for(i = 0; i<lengthnum; i++)
{
    for(j = 0; j<COUNTNUM; j++)
    {
        if(averagelist[i*COUNTNUM+j] < 0.0001) averagelist[i*COUNTNUM+j] = 0.0001;
	averagelist[i*COUNTNUM+j] /= lengthcounts[i];
    }
}


fstream outFile;

outFile.open("counts.txt", fstream::out);
if(!outFile) die("Failed to properly open file for output. Aborting.");

outFile << "These are the counts for the advanced algorithms (both recursive and iterative share more or less the same statistics,\n";
outFile << "they only differ in the order of carrying out computations), and the initial algorithm which is less advanced.\n";
outFile << "For ease of use, they are output already in Mathematica format.\n\n";

outFile << "Printing the number of ppsumred calls (the number of times a partition sum was required)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i  ];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of ppsum calls (the number of partition sums that have to be calculated at least)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+1];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of ppsumarc calls (this is the number of different arcs possible on the strand)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+2];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of flops (the number of multiplications between two doubles)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+3];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of prevented calls (the number of times ppsumred prevented a call to ppsum)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+4];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of array lookups (the number of times we had to read from or write into an array)\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+5];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of flops (the number of multiplications between two doubles) for the initial algorithm \n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+6];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Unused position... \n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+7];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}
outFile << "Printing the number of array lookups (the number of times we had to read from or write into an array) for the initial algorithm\n{";
for(i = 0; i<lengthnum; i++)
{
    outFile << "{"<<strlengtharray[i]<<","<<averagelist[COUNTNUM*i+8];
    if(i<lengthnum-1) outFile << "},";
    else outFile << "}}\n\n";
}

cout << "Everything written to file. Closing.\n";
outFile.close();

delete [] calls;
delete [] strlengtharray;
delete [] averagelist;
delete [] lengthcounts;
}

// Main function.
int main(void)
{
    cout << "Algorithm testing program starting...\n";
    runTest();
    cout << "Finished calculations.\n";
    return 1;
}
